% Last Updated. October 2019, Jacob Schwartz and Nathan Canen.

% M99_Simulations_Simple_Type.m is the main program file for conducting simulation
% experiments investigating the finite sample size and power properties of
% the asymptotic inference proposed in Canen, Schwartz, and Song (2019) for
% a variety of graph configurations.

% Given parameters, this file generates saves these parameters along with
% the following in a text (.txt) file : empirical coverage probabilities
% for beta and a'rho average length of confidence intervals for beta and
% a'rho


% graph_generate.m
%
%           Generates either Barabasi-Albert (BA) if gtype=0 or Erdos Renyi
%           (ER) graph if gtype=1 Calls barabasi_albert.m, main_ba_graph.m,
%           and erdos_renyi.m if gtype=0 Calls erdos_renyi.m if gtype=1

% barabasi_albert.m,  main_ba_graph.m
%
%           Generate BA graph of size n and parameter m=lam given initial
%           ER seed graph when gtype=0. To change the size of seed graph,
%           modify these files.

% cij_gen_2019.m
%
%           Generate cij's given a payoff graph

% erdos_renyi.m
%            Generate ER graph of size n and parameter lambda=lam (gtype=1)

% lambda_gen_sims_2019.m
%           Generates lambda_ij matrix (defined in Section 2.3.)


% q_e_ij_gen.m,
%           Calculates q_e_ij's

% stats_asymptotic.m
%           Calculates value of test statistics from asymptotic inference
%           given the simulation data
%%

clear
seed=17293912;
rng(seed,'twister')



for gtype = 0:1; %Loop over graph specifications
    
    %Set up Monte Carlo Study
    
    y_under_null=0; %1 we generate Y under the null of no info sharing
    n_mc=[500,1000,5000]'; %Set of graph sizes to explore
    lam_mc=[1,2,3]'; %Set of graph parameters to explore
    %Then "lam" will be associated with either the lambda parameter in the
    %Erdos-Renyi model or the m parameter of Barabasi-Albert
    
    nms=size(n_mc,1); % Number of different sample sizes in the MC study
    lams=size(lam_mc,1); % Number of graph parameters for each sample size in MC study
    
    R=5000; % Simulation Number
    alpha=0.05; %nominal si;
    dw=1;
    d1=3; %gamma must have d1 elements
    d2=2; %delta must have d2 elements
    
    gamma_0=[2,4,1]';
    delta_0=[3,4]';
    rho_0=[gamma_0;delta_0];
    
    d=d1+d2; % Dimension of rho
    a_vec=ones(d,1); %We wish to perform inference on a'rho.
    
    %Values of beta_0 to consider in simulation studies
    beta_0_grid=[-0.5,-0.3,0,0.3,0.5]';
    beta_0_grid_s=size(beta_0_grid,1);
    
    %Next we construct the set of alternatives for beta. Please ensure that
    %beta_1_grid is constructed so that the elements of beta_0_grid are
    %contained within it
    
    stepsize=0.025;
    ub=0.95; %Upper bound of parameter space for beta
    lb=-0.95; %Lower bound for parameter space for beta
    ttl_length=0.5; %length of alternative interval for each beta_0
    beta_1_grid=[lb:stepsize:ub]';    beta_1_grid_s=size(beta_1_grid,1);
    
    %Moments of X, epsilons, etas
    mu_x1=1;    mu_x1m=mu_x1*ones(1,d1);    sig_x1=eye(d1);    mu_x2=3;    mu_x2m=mu_x2*ones(1,d2);    sig_x2=eye(d2);
    mu_alpha0=0;    sig_alpha0=1;    mu_epsi=0;    sig_epsi=1;    mu_eta=0;    sig_eta=1;
    
    % Generate All Graphs for Monte Carlo study
    Gp=cell(nms,lams);    Gp2=cell(nms,lams);    Np=cell(nms,lams);    Np2=cell(nms,lams);    nP=cell(nms,lams);    nP2=cell(nms,lams);    Gp2_tilde=cell(nms,lams);
    Np2_tilde=cell(nms,lams);    nP2_tilde=cell(nms,lams);    ci_vectors=cell(nms,lams);    cij_matrices=cell(nms,lams);    lam_list=cell(nms,lams);
    
    %Generate graph related objects
    disp('Generating graph materials')
    for nn=1:nms %Checked Nov 24, 2016, Feb 6, 2017
        n=n_mc(nn);
        for ll=1:lams;
            lam=lam_mc(ll);
            [n,lam]
            graphdata=graph_generate(lam,n,gtype);
            Gp_temp=graphdata{1};
            Gp{nn,ll}=sparse(Gp_temp);
            Np{nn,ll}=graphdata{2};
            nP{nn,ll}=graphdata{3};
            Gp2_temp=graphdata{4};
            Gp2{nn,ll}=sparse(Gp2_temp);
            Np2{nn,ll}=graphdata{5};
            nP2{nn,ll}=graphdata{6};
            Gp2_tilde_temp=graphdata{7};
            Gp2_tilde{nn,ll}=sparse(Gp2_tilde_temp);
            Np2_tilde{nn,ll}=graphdata{8};
            nP2_tilde{nn,ll}=graphdata{9};
            lam_list{nn,ll}=lam;
            cij_temp=cij_gen_2019(n,nP{nn,ll},Np{nn,ll}); 
            cij_matrices{nn,ll}=sparse(cij_temp);
        end
    end
       
    %Put graph data in strucutre
    graph_=struct('Gp',Gp,'Np',Np,'nP',nP,'Gp2',Gp2,'Np2',Np2,'nP2',nP2,'Gp2_tilde',Gp2_tilde,'Np2_tilde',Np2_tilde,'nP2_tilde',nP2_tilde,...
        'lam',lam_list,'ci_vectors',ci_vectors,'cij_matrices',cij_matrices);
    
    
    s = RandStream('mt19937ar','Seed',1);
    RandStream.setGlobalStream(s);
    
    beta_1_grid_b0=[0-ttl_length/2:stepsize:0+ttl_length/2]';
    beta_1_grid_b0_s=size(beta_1_grid_b0,1);
    
    non_reject_prob_betas=zeros(beta_1_grid_b0_s,beta_0_grid_s,nms,lams);
    non_reject_prob_ap_rho=zeros(beta_0_grid_s,nms,lams);
    av_length_betas=zeros(beta_0_grid_s,nms,lams);
    reject_prob_IU=zeros(beta_0_grid_s,nms,lams);
    rp_beta_known=zeros(beta_1_grid_b0_s,beta_0_grid_s,nms,lams);
    av_length_ap_rho=zeros(beta_0_grid_s,nms,lams);
    
    
    for nn=1:nms % Loop over sample size for simulations
        
        n=n_mc(nn);
        zer_n_by_n=zeros(n,n);
        zer_1_by_n=zeros(1,n);
        zer_n_by_1=zeros(n,1);
        
        z_=struct('zer_n_by_n',zer_n_by_n,...
            'zer_n_by_1',zer_n_by_1,'zer_1_by_n',zer_1_by_n);
       
        %Generate X1, X2, etas, epsilons, and phis for each of the R
        %simulations
        
        
        for ll=1:lams   % Loop over graph parameter for simulations
             
        one=ones(1,n);        sqrtn=sqrt(n);        X1s=cell(R,1);        X2s=cell(R,1);        X2bars=cell(R,1);        Xs=cell(R,1);
        etas=cell(R,1);        epsis=cell(R,1);     phi_x_term=cell(R,1);
        one_n=ones(n,1);
        
            
            for r=1:R
                X1=mvnrnd(mu_x1m,sig_x1,n);
                X1(:,1)=one_n;
                X1s{r}=X1;
                X2s{r}=mvnrnd(mu_x2m,sig_x2,n);
                X2bars{r}=(graph_(nn,ll).Gp*X2s{r}).*((1./graph_(nn,ll).nP)*ones(1,d2));
                X2bars{r}(isnan(X2bars{r}))=0;
                Xs{r}=[X1s{r},X2bars{r}];
                etas{r}=normrnd(mu_eta,sig_eta,[n,1]);
                epsis{r}=normrnd(mu_epsi,sig_epsi,[n,1]);
                if y_under_null==1
                    epsis{r}=zeros(n,1);
                else
                    epsis{r}=normrnd(mu_epsi,sig_epsi,[n,1]);
                end
                phi_x_term{r}=[X1s{r}.^2, X2bars{r}.^2, X2bars{r}.^3];          %September 2019 Question: Does this still make sense
            end
            
            X_e_=struct('Xs',Xs,'etas',etas,'epsis',epsis,'X1s',X1s);    %Store random draws in structure
            
            clear X1s X2s X2bars Xs etas epsis
            M=d1+size(phi_x_term{1},2);
            
            lam=lam_mc(ll);
            
            Test_statistics_beta=cell(beta_1_grid_b0_s,R,beta_0_grid_s);
            C_beta_1_minus_alpha=zeros(beta_1_grid_b0_s,R,beta_0_grid_s);
            C_beta_1_minus_kappa=zeros(beta_1_grid_b0_s,R,beta_0_grid_s);
            C_beta_rho=zeros(2,beta_1_grid_b0_s,R,beta_0_grid_s);
            IU_betas=zeros(beta_1_grid_b0_s,R,beta_0_grid_s);
            IU_betas_beta_known=zeros(beta_1_grid_b0_s,R,beta_0_grid_s);
            zeros_mm=zeros(M,M);  om=ones(1,M);  eyem=eye(M); zero=zeros(n,1);
            critval_beta=chi2inv(1-alpha,M-(d1+d2));
            critval_rho=norminv(1-alpha/4,0,1);
            critval_alpha_over_two=chi2inv(1-alpha/2,M-(d1+d2));
            critval_IU=chi2inv(1-(alpha)/2,1);
            critval_IU_beta_known=chi2inv(1-(alpha),1);
            weight=sqrt((graph_(nn,ll).nP+1))*om; % These are the weights for downweighting.
            
            num_dig = 4;
            sig_figs=10^num_dig;
            beta_1_grid=round(beta_1_grid*(sig_figs))/(sig_figs);
            
            ar_=struct('zeros_mm',zeros_mm,'om',om,'eyem',eyem,'n',n,'one',one,'zero',zero,'sqrtn',sqrtn,'a_vec',a_vec,'critval_rho',critval_rho);
            
            for bb_0=1:beta_0_grid_s; %Loop over true beta
                
                
                beta_0=beta_0_grid(bb_0);
                beta_1_grid_b0=[beta_0-ttl_length/2:stepsize:beta_0+ttl_length/2]';
                beta_1_grid_b0=round(beta_1_grid_b0*(sig_figs))/(sig_figs);                
                out=lambda_gen_sims_2019(n,nn,ll,graph_,z_,beta_0);
                lambda_0_matrix=out{1};
                lambda_0_matrix_np=out{2};
                lambda_0_bars=out{3};
                sum_np =(lambda_0_matrix_np.*graph_(nn,ll).Gp);
                w0 = one_n+beta_0^2*lambda_0_bars./(graph_(nn,ll).nP-beta_0^2*lambda_0_bars);
                w0(isnan(w0))=0; %Because empty payoff neighborhood
                w0vec=w0*ones(1,d);
                
                for bb_1=1:beta_1_grid_b0_s %Loop over alternative beta
                    beta_1=beta_1_grid_b0(bb_1);
                    out=lambda_gen_sims_2019(n,nn,ll,graph_,z_,beta_1);
                    lambda_1_matrix=out{1};
                    lambda_1_matrix_np=out{2};
                    lambda_1_bars=out{3};
                    sum_np_1 =(lambda_1_matrix_np.*graph_(nn,ll).Gp);
                    w0_1 = one_n+beta_1^2*lambda_1_bars./(graph_(nn,ll).nP-beta_1^2*lambda_1_bars);
                    w0_1(isnan(w0_1))=0; %Because empty payoff neighborhood
                    w0vec_1=w0_1*ones(1,d);
                    q_e_ij_s=q_e_ij_gen(n,nn,ll,graph_,z_,beta_1,lambda_1_matrix,w0_1);
                    q_e_=struct('q_e_ij_s',q_e_ij_s);
                    
                    %[Sample size, Graph Parameter, True Beta, Alternative
                    %Beta]
                    [n,lam,beta_0,beta_1]
                    
                    parfor r=1:R
                        
                        %GENERATE INSTRUMENT
                        Z_1=((X_e_(r).Xs+beta_1*sum_np_1*X_e_(r).Xs)).*w0vec_1;
                        phi_0=[sum_np_1*X_e_(r).X1s, phi_x_term{r}];
                        %Downweighting: dw=1 if downweighting, 0 otherwise.
                        if dw==1
                            phi_tilde=phi_0./weight;
                        else
                            phi_tilde=phi_0;
                        end
                        %GENERATE DEPENDENT VARIABLE
                        Z_0=((X_e_(r).Xs+beta_0*sum_np*X_e_(r).Xs)).*w0vec;
                        V_0=((X_e_(r).epsis+beta_0*sum_np*X_e_(r).epsis)).*w0+X_e_(r).etas;
                        Y_0=Z_0*rho_0+V_0;
                        %Compute test statistics for asymptotic inference
                        %on the parameters
                        stat_out=stats_asymptotic(nn,ll,graph_,Y_0,phi_tilde,Z_1,q_e_,ar_);
                        Test_statistics_beta{bb_1,r,bb_0}=stat_out{1};
                        C_beta_1_minus_alpha(bb_1,r,bb_0)=Test_statistics_beta{bb_1,r,bb_0}<critval_beta;
                        C_beta_1_minus_alpha_over_two(bb_1,r,bb_0)=Test_statistics_beta{bb_1,r,bb_0}<critval_alpha_over_two;
                        C_beta_rho(:,bb_1,r,bb_0)=(stat_out{2})';
                        IU_betas(bb_1,r,bb_0)=stat_out{3};
                        IU_betas_beta_known(bb_1,r,bb_0)=stat_out{3}<critval_IU_beta_known;
                    end
                end
            end
            
            % Compute average lengths of confidence intervals and rejection
            % probabilities
            tru_param=a_vec'*rho_0;
            C_beta_rho_p=zeros(R,beta_0_grid_s);
            length_arho=zeros(R,beta_0_grid_s);
            length_beta=zeros(R,beta_0_grid_s);
            C_beta_rho=real(C_beta_rho);
            IU=zeros(R,beta_0_grid_s);
            
            %Compute the length of C^{\rho}_{1-\alpha}(a)
            for bb_0=1:beta_0_grid_s;
                
                for r=1:R
                    %Count the number of points in the confidence region
                    %for beta in simulation r
                    %             ns=sum(C_beta_1_minus_kappa(:,r,bb_0));
                    Intervals=(C_beta_rho(:,:,r,bb_0))'.*(C_beta_1_minus_alpha_over_two(:,r,bb_0)*ones(1,2));
                    %The union of these intervals is equal to
                    %C^\rho_1-\alpha(a)
                    Intervals( all(~Intervals,2), : ) = [];
                    ns2=size(Intervals,1);
                    
                    if isempty(Intervals)==0
                        
                        
                        length_arho(r,bb_0)=max(max(Intervals))-min(min(Intervals));
                        
                        contains=0;
                        for l=1:ns2
                            contains=contains+(Intervals(l,1) <= tru_param)*(tru_param <= Intervals(l,2));
                        end
                        C_beta_rho_p(r,bb_0)=1*(contains>0);
                        
                        %Check if C^\rho_1-\alpha covers the true parameter
                        %value.
                    else
                        length_arho(r,bb_0)=NaN;
                        C_beta_rho_p(r,bb_0)=0;
                        
                    end
                    
                    %Compute the length for beta confidence interval
                    indices=find(C_beta_1_minus_alpha(:,r,bb_0));
                    min_index=min(indices);
                    max_index=max(indices);
                    if isempty(indices)==0
                        length_beta(r,bb_0)=beta_1_grid(max_index)-beta_1_grid(min_index);
                    else
                        length_beta(r,bb_0)=NaN;
                    end
                    select_IU_beta=C_beta_1_minus_alpha_over_two(:,r,bb_0).*IU_betas(:,r,bb_0);
                    select_IU_beta(all(~select_IU_beta,2), : )=[];
                    if isempty(select_IU_beta)==1
                        IU(r,bb_0)=NaN;
                    else
                        IU(r,bb_0)=min(select_IU_beta);
                    end
                    
                end
            end
            
            % Process average lengths of confidence intervals
            av_length_beta=zeros(beta_0_grid_s,1);
            av_length_arho=zeros(beta_0_grid_s,1);
            for bb_0=1:beta_0_grid_s;
                lbetatemp=length_beta(:,bb_0);
                lbetatemp2=lbetatemp(~any(isnan(lbetatemp),2),:);
                if isempty(lbetatemp2)==1
                    av_length_beta(bb_0)=NaN;
                else
                    av_length_beta(bb_0)=mean(lbetatemp2);
                end
                larhotemp=length_arho(:,bb_0);
                larhotemp2=larhotemp(~any(isnan(larhotemp),2),:);
                if isempty(larhotemp2)==1
                    av_length_arho(bb_0)=NaN;
                else
                    av_length_arho(bb_0)=mean(larhotemp2);
                end
                reject_prob_IU(bb_0,nn,ll)=1-nanmean(IU(:,bb_0)<critval_IU);
            end
            rp_beta_known(:,:,nn,ll)=1-squeeze(nanmean(IU_betas_beta_known,2));
            non_reject_prob_betas(:,:,nn,ll)=squeeze(mean(C_beta_1_minus_alpha,2));
            non_reject_prob_ap_rho(:,nn,ll)=mean(C_beta_rho_p);
            av_length_betas(:,nn,ll)=av_length_beta;
            av_length_ap_rho(:,nn,ll)=av_length_arho;
        end
    end
    
    
    
    
    %% Store simulation output
    
    % Make a text file with the finite sample non-rejection probabilities
    % and average lengths of confidence intervals
    
    diary Simulation_Results_Simple_Type.txt
    
    display('beta_0')
    
    disp('Non-rejection probabilities, beta inference')
    non_reject_prob_betas
    disp('Non-rejection probabilities, ap rho inference')
    non_reject_prob_ap_rho
    disp('Average length CI, beta inference')
    av_length_betas
    disp('Average length CI, ap rho inference')
    av_length_ap_rho
    
    dw       % downweighting used if dw=1
    lam_mc   % graph parameters
    n_mc     % sample size
    gtype    % gtype: Select the model for payoff graph, 1 for ER, 0 for BA;
    a_vec    % rho inference weight
    alpha    % alpha
    R        % simulation number
    
    %Rejection probability for IU Statistic
    reject_prob_IU
    
    diary off
    
    %Store statistics (max degree, average degree) of graphs used in
    %simulation study
    
    diary Graph_Statistics_Simple_Type.txt
    gtype
    for nn=1:nms
        n=n_mc(nn);
        for ll=1:lams;
            lam=lam_mc(ll);
            disp('graph size, graph param, max degree, av, degree')
            [n,lam]
            max(sum(nP{nn,ll},2))
            mean(sum(nP{nn,ll},2))
        end
    end
    
    
    diary off
    
    
    
end





